package javax.swing.text;

import javax.swing.event.DocumentEvent;
import sun.swing.SwingUtilities2;
import javax.swing.JComponent;
import java.awt.Graphics;
import javax.swing.BoundedRangeModel;
import java.awt.Rectangle;
import javax.swing.JTextField;
import java.awt.Shape;
import java.awt.Container;
import java.awt.FontMetrics;

public class FieldView extends PlainView
{
    public FieldView(Element element) {
        super(element);
    }
    
    protected FontMetrics getFontMetrics() {
        Container container = this.getContainer();
        return container.getFontMetrics(container.getFont());
    }
    
    protected Shape adjustAllocation(Shape shape) {
        if (shape != null) {
            Rectangle bounds = shape.getBounds();
            int n = (int)this.getPreferredSpan(1);
            int n2 = (int)this.getPreferredSpan(0);
            if (bounds.height != n) {
                int n3 = bounds.height - n;
                bounds.y += n3 / 2;
                bounds.height -= n3;
            }
            Container container = this.getContainer();
            if ((container instanceof JTextField)) {
                BoundedRangeModel horizontalVisibility = ((JTextField)container).getHorizontalVisibility();
                int max = Math.max(n2, bounds.width);
                int value = horizontalVisibility.getValue();
                int min = Math.min(max, bounds.width - 1);
                if (value + min > max) {
                    value = max - min;
                }
                horizontalVisibility.setRangeProperties(value, min, horizontalVisibility.getMinimum(), max, false);
                if (n2 < bounds.width) {
                    int n4 = bounds.width - 1 - n2;
                    int horizontalAlignment = ((JTextField)container).getHorizontalAlignment();
                    if (Utilities.isLeftToRight(container)) {
                        if (horizontalAlignment == 10) {
                            horizontalAlignment = 2;
                        }
                        else if (horizontalAlignment == 11) {
                            horizontalAlignment = 4;
                        }
                    }
                    else if (horizontalAlignment == 10) {
                        horizontalAlignment = 4;
                    }
                    else if (horizontalAlignment == 11) {
                        horizontalAlignment = 2;
                    }
                    switch (horizontalAlignment) {
                        case 0: {
                            bounds.x += n4 / 2;
                            bounds.width -= n4;
                            break;
                        }
                        case 4: {
                            bounds.x += n4;
                            bounds.width -= n4;
                            break;
                        }
                    }
                }
                else {
                    bounds.width = n2;
                    bounds.x -= horizontalVisibility.getValue();
                }
            }
            return bounds;
        }
        else {
            return null;
        }
    }
    
    void updateVisibilityModel() {
        Container container = this.getContainer();
        if ((container instanceof JTextField)) {
            BoundedRangeModel horizontalVisibility = ((JTextField)container).getHorizontalVisibility();
            int a = (int)this.getPreferredSpan(0);
            int extent = horizontalVisibility.getExtent();
            int max = Math.max(a, extent);
            int n = (extent == 0) ? max : extent;
            int a2 = max - n;
            int value = horizontalVisibility.getValue();
            if (value + n > max) {
                value = max - n;
            }
            horizontalVisibility.setRangeProperties(Math.max(0, Math.min(a2, value)), n, 0, max, false);
        }
    }
    
    @Override
    public void paint(Graphics graphics, Shape shape) {
        Rectangle rectangle = (Rectangle)shape;
        graphics.clipRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
        super.paint(graphics, shape);
    }
    
    @Override
    Shape adjustPaintRegion(Shape shape) {
        return this.adjustAllocation(shape);
    }
    
    @Override
    public float getPreferredSpan(int n) {
        switch (n) {
            case 0: {
                Segment sharedSegment = SegmentCache.getSharedSegment();
                Document document = this.getDocument();
                int tabbedTextWidth;
                try {
                    FontMetrics fontMetrics = this.getFontMetrics();
                    document.getText(0, document.getLength(), sharedSegment);
                    tabbedTextWidth = Utilities.getTabbedTextWidth(sharedSegment, fontMetrics, 0, this, 0);
                    if (sharedSegment.count > 0) {
                        Container container = this.getContainer();
                        this.firstLineOffset = SwingUtilities2.getLeftSideBearing((container instanceof JComponent) ? ((JComponent)container) : null, fontMetrics, sharedSegment.array[sharedSegment.offset]);
                        this.firstLineOffset = Math.max(0, -this.firstLineOffset);
                    }
                    else {
                        this.firstLineOffset = 0;
                    }
                }
                catch (BadLocationException ex) {
                    tabbedTextWidth = 0;
                }
                SegmentCache.releaseSharedSegment(sharedSegment);
                return (float)(tabbedTextWidth + this.firstLineOffset);
            }
            default: {
                return super.getPreferredSpan(n);
            }
        }
    }
    
    @Override
    public int getResizeWeight(int n) {
        if (n == 0) {
            return 1;
        }
        return 0;
    }
    
    @Override
    public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
        return super.modelToView(n, this.adjustAllocation(shape), bias);
    }
    
    @Override
    public int viewToModel(float n, float n2, Shape shape, Position.Bias[] array) {
        return super.viewToModel(n, n2, this.adjustAllocation(shape), array);
    }
    
    @Override
    public void insertUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        super.insertUpdate(documentEvent, this.adjustAllocation(shape), viewFactory);
        this.updateVisibilityModel();
    }
    
    @Override
    public void removeUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        super.removeUpdate(documentEvent, this.adjustAllocation(shape), viewFactory);
        this.updateVisibilityModel();
    }
}
